defmodule Hafnium.Compile do
  @moduledoc false

  alias Hafnium.Pipeline
  alias Hafnium.Utils

  def build_pipeline!(module) when is_atom(module), do: build_pipeline!({module, []})

  def build_pipeline!({module, opts}) when is_atom(module) do
    unless Utils.defined?(module) do
      raise CompileError, description: "`#{inspect(module)}` should be a module"
    end

    behaviours =
      module.module_info()[:attributes]
      |> Keyword.take([:behaviour])
      |> Keyword.values()
      |> List.flatten()

    behavior_module = Module.concat(module, PipelineBehaviour)

    behaviours
    |> Enum.any?(&match?(^behavior_module, &1))
    |> unless do
      raise CompileError,
        description: "`#{inspect(module)}` should use #{behavior_module}"
    end

    kind =
      cond do
        Enum.any?(behaviours, &match?(GenServer, &1)) -> :gen_server
        true -> :normal
      end

    %Pipeline{__module__: module, kind: kind, input: opts}
  end
end
