defmodule Hafnium.Pipeline do
  @moduledoc """
  Pipeline
  """

  alias Hafnium.Utils

  @type kind :: :normal | :gen_server

  @type t :: %{
          __module__: module(),
          kind: kind(),
          input: Hafnium.input(),
          index: non_neg_integer(),
          start_time: integer(),
          end_time: integer(),
          result: Hafnium.result()
        }

  @enforce_keys [:__module__]
  defstruct [:index, :kind, :__module__, input: %{}]

  defmacro __using__(_opts) do
    quote location: :keep do
      unless Module.get_attribute(__MODULE__, :moduledoc) do
        @moduledoc """
        See `#{unquote(__MODULE__)}`.
        """
      end

      def before_call(o), do: o
      def after_call(o), do: o

      defoverridable before_call: 1, after_call: 1

      @before_compile unquote(__MODULE__)

      import unquote(__MODULE__)

      alias __MODULE__.PipelineBehaviour
      @behaviour PipelineBehaviour
    end
  end

  defmacro __before_compile__(env) do
    quote location: :keep, generated: true do
      defmodule PipelineBehaviour do
        @moduledoc false
        @callback call(input :: Hafnium.t()) :: Hafnium.pipe_result()
        @callback before_call(input :: Hafnium.t()) :: Hafnium.pipe_result()
        @callback after_call(input :: Hafnium.t()) :: Hafnium.pipe_result()
      end

      @spec name :: atom()
      def name, do: unquote(Utils.module_name(env.module))
    end
  end
end
